@@ -19,7 +19,7 @@
  *
  * This file implements the basic IO handling for the mkII protocol.
  *
- * $Id: jtag2io.cc,v 1.12 2007/10/29 23:31:05 joerg_wunsch Exp $
+ * $Id: jtag2io.cc,v 1.14 2008/06/12 20:21:44 joerg_wunsch Exp $
  */
 
 
@@ -101,8 +101,8 @@ int jtag2::recvFrame(unsigned char *&msg, unsigned short &seqno)
 	sSTART, sSEQNUM1, sSEQNUM2, sSIZE1, sSIZE2, sSIZE3, sSIZE4,
 	sTOKEN, sDATA, sCSUM1, sCSUM2, sDONE
     }  state = sSTART;
-    int msglen = 0, l = 0;
-    int headeridx = 0;
+    unsigned int msglen = 0, l = 0;
+    unsigned int headeridx = 0;
     bool ignorpkt = false;
     int rv;
     unsigned char c, *buf = NULL, header[8];
@@ -233,7 +233,7 @@ int jtag2::recvFrame(unsigned char *&msg, unsigned short &seqno)
     seqno = r_seqno;
     msg = buf;
 
-    return msglen;
+    return (int)msglen;
 }
 
 /*
@@ -492,7 +492,14 @@ void jtag2::startJtagLink(void)
 
     for (unsigned int i = 0; i < sizeof bitrates / sizeof *bitrates; i++)
 	if (synchroniseAt(bitrates[i])) {
-	    uchar val = useDebugWire? EMULATOR_MODE_DEBUGWIRE: EMULATOR_MODE_JTAG;
+	    uchar val;
+
+	    if (apply_nSRST) {
+		val = 0x01;
+		setJtagParameter(PAR_EXTERNAL_RESET, &val, 1);
+	    }
+
+	    val = useDebugWire? EMULATOR_MODE_DEBUGWIRE: EMULATOR_MODE_JTAG;
 	    setJtagParameter(PAR_EMULATOR_MODE, &val, 1);
 	    signedIn = true;
 
