@@ -19,7 +19,7 @@
  *
  * This file implements target execution handling for the mkII protocol.
  *
- * $Id: jtag2run.cc,v 1.9 2007/10/29 23:31:05 joerg_wunsch Exp $
+ * $Id: jtag2run.cc,v 1.13 2008/06/12 20:21:44 joerg_wunsch Exp $
  */
 
 
@@ -73,7 +73,7 @@ bool jtag2::setProgramCounter(unsigned long pc)
     return true;
 }
 
-bool jtag2::resetProgram(void)
+bool jtag2::resetProgram(bool possible_nSRST_ignored)
 {
     if (useDebugWire) {
 	/* The JTAG ICE mkII and Dragon do not respond correctly to
@@ -88,6 +88,9 @@ bool jtag2::resetProgram(void)
 	bool rv = doJtagCommand(cmd, 2, resp, respSize);
 	delete [] resp;
 
+	/* Await the BREAK event that is posted by the ICE. */
+	(void)eventLoop();
+
 	return rv;
     }
 }
@@ -111,6 +114,150 @@ bool jtag2::resumeProgram(void)
     return true;
 }
 
+bool jtag2::eventLoop(void)
+{
+    int maxfd;
+    fd_set readfds;
+    bool breakpoint = false, gdbInterrupt = false;
+
+    // Now that we are "going", wait for either a response from the JTAG
+    // box or a nudge from GDB.
+
+    if (ctrlPipe != -1)
+      {
+	  /* signal the USB daemon to start polling. */
+	  char cmd[1] = { 'p' };
+	  (void)write(ctrlPipe, cmd, 1);
+      }
+
+    for (;;)
+      {
+	  debugOut("Waiting for input.\n");
+
+	  // Check for input from JTAG ICE (breakpoint, sleep, info, power)
+	  // or gdb (user break)
+	  FD_ZERO (&readfds);
+	  if (gdbFileDescriptor != -1)
+	    FD_SET (gdbFileDescriptor, &readfds);
+	  FD_SET (jtagBox, &readfds);
+	  if (gdbFileDescriptor != -1)
+	    maxfd = jtagBox > gdbFileDescriptor ? jtagBox : gdbFileDescriptor;
+	  else
+	    maxfd = jtagBox;
+
+	  int numfds = select(maxfd + 1, &readfds, 0, 0, 0);
+	  unixCheck(numfds, "GDB/JTAG ICE communications failure");
+
+	  if (gdbFileDescriptor != -1 && FD_ISSET(gdbFileDescriptor, &readfds))
+	    {
+		int c = getDebugChar();
+		if (c == 3) // interrupt
+		  {
+		      debugOut("interrupted by GDB\n");
+		      gdbInterrupt = true;
+		  }
+		else
+		    debugOut("Unexpected GDB input `%02x'\n", c);
+	    }
+
+	  if (FD_ISSET(jtagBox, &readfds))
+	    {
+		uchar *evtbuf;
+		int evtSize;
+		unsigned short seqno;
+		evtSize = recvFrame(evtbuf, seqno);
+		if (evtSize >= 0) {
+		    // XXX if not event, should push frame back into queue...
+		    // We really need a queue of received frames.
+		    if (seqno != 0xffff)
+			debugOut("Expected event packet, got other response");
+		    else if (!nonbreaking_events[evtbuf[8] - EVT_BREAK])
+		      {
+			  switch (evtbuf[8])
+			    {
+				// Program stopped at some kind of breakpoint.
+			    case EVT_BREAK:
+			    case EVT_EXT_RESET:
+			    case EVT_PDSB_BREAK:
+			    case EVT_PDSMB_BREAK:
+			    case EVT_PROGRAM_BREAK:
+				breakpoint = true;
+				break;
+
+			    case EVT_IDR_DIRTY:
+				// The program is still running at IDR dirty, so
+				// pretend a user break;
+				gdbInterrupt = true;
+				printf("\nIDR dirty: 0x%02x\n", evtbuf[9]);
+				break;
+
+				// Fatal debugWire errors, cannot continue
+			    case EVT_ERROR_PHY_FORCE_BREAK_TIMEOUT:
+			    case EVT_ERROR_PHY_MAX_BIT_LENGTH_DIFF:
+			    case EVT_ERROR_PHY_OPT_RECEIVE_TIMEOUT:
+			    case EVT_ERROR_PHY_OPT_RECEIVED_BREAK:
+			    case EVT_ERROR_PHY_RECEIVED_BREAK:
+			    case EVT_ERROR_PHY_RECEIVE_TIMEOUT:
+			    case EVT_ERROR_PHY_RELEASE_BREAK_TIMEOUT:
+			    case EVT_ERROR_PHY_SYNC_OUT_OF_RANGE:
+			    case EVT_ERROR_PHY_SYNC_TIMEOUT:
+			    case EVT_ERROR_PHY_SYNC_TIMEOUT_BAUD:
+			    case EVT_ERROR_PHY_SYNC_WAIT_TIMEOUT:
+				gdbInterrupt = true;
+				printf("\nFatal debugWIRE communication event: 0x%02x\n",
+				       evtbuf[8]);
+				break;
+
+				// Other fatal errors, user could mask them off
+			    case EVT_ICE_POWER_ERROR_STATE:
+				gdbInterrupt = true;
+				printf("\nJTAG ICE mkII power failure\n");
+				break;
+
+			    case EVT_TARGET_POWER_OFF:
+				gdbInterrupt = true;
+				printf("\nTarget power turned off\n");
+				break;
+
+			    case EVT_TARGET_POWER_ON:
+				gdbInterrupt = true;
+				printf("\nTarget power returned\n");
+				break;
+
+			    case EVT_TARGET_SLEEP:
+				gdbInterrupt = true;
+				printf("\nTarget went to sleep\n");
+				break;
+
+			    case EVT_TARGET_WAKEUP:
+				gdbInterrupt = true;
+				printf("\nTarget went out of sleep\n");
+				break;
+
+				// Events where we want to continue
+			    case EVT_NONE:
+			    case EVT_RUN:
+				break;
+
+			    default:
+				gdbInterrupt = true;
+				printf("\nUnhandled JTAG ICE mkII event: 0x%0x2\n",
+				       evtbuf[8]);
+			    }
+		      }
+		    delete [] evtbuf;
+		}
+	    }
+
+	  // We give priority to user interrupts
+	  if (gdbInterrupt)
+	      return false;
+	  if (breakpoint)
+	      return true;
+      }
+}
+
+
 bool jtag2::jtagSingleStep(bool useHLL)
 {
     uchar cmd[3] = { CMND_SINGLE_STEP,
@@ -134,7 +281,12 @@ bool jtag2::jtagSingleStep(bool useHLL)
     }
     while (--i >= 0);
 
-    return rv;
+    if (!rv)
+        return false;
+
+    (void)eventLoop();
+
+    return true;
 }
 
 void jtag2::parseEvents(const char *evtlist)
@@ -224,79 +376,6 @@ bool jtag2::jtagContinue(void)
     else
 	doSimpleJtagCommand(CMND_GO);
 
-    for (;;)
-    {
-	int maxfd;
-	fd_set readfds;
-	bool breakpoint = false, gdbInterrupt = false;
-
-	// Now that we are "going", wait for either a response from the JTAG
-	// box or a nudge from GDB.
-	debugOut("Waiting for input.\n");
-
-	if (ctrlPipe != -1)
-	  {
-	    /* signal the USB daemon to start polling. */
-	    char cmd[1] = { 'p' };
-	    (void)write(ctrlPipe, cmd, 1);
-	  }
-
-	// Check for input from JTAG ICE (breakpoint, sleep, info, power)
-	// or gdb (user break)
-	FD_ZERO (&readfds);
-	FD_SET (gdbFileDescriptor, &readfds);
-	FD_SET (jtagBox, &readfds);
-	maxfd = jtagBox > gdbFileDescriptor ? jtagBox : gdbFileDescriptor;
-
-	int numfds = select(maxfd + 1, &readfds, 0, 0, 0);
-	unixCheck(numfds, "GDB/JTAG ICE communications failure");
-
-	if (FD_ISSET(gdbFileDescriptor, &readfds))
-	{
-	    int c = getDebugChar();
-	    if (c == 3) // interrupt
-	    {
-		debugOut("interrupted by GDB\n");
-		gdbInterrupt = true;
-	    }
-	    else
-		debugOut("Unexpected GDB input `%02x'\n", c);
-	}
-
-	if (FD_ISSET(jtagBox, &readfds))
-	{
-	    uchar *evtbuf;
-	    int evtSize;
-	    unsigned short seqno;
-	    evtSize = recvFrame(evtbuf, seqno);
-	    if (evtSize >= 0) {
-		// XXX if not event, should push frame back into queue...
-		// We really need a queue of received frames.
-		if (seqno != 0xffff)
-		    debugOut("Expected event packet, got other response");
-		else if (!nonbreaking_events[evtbuf[8] - EVT_BREAK])
-                {
-                    if (evtbuf[8] == EVT_IDR_DIRTY)
-                    {
-                        // The program is still running at IDR dirty, so
-                        // pretend a user break;
-                        gdbInterrupt = true;
-                        printf("\nIDR dirty: 0x%02x\n", evtbuf[9]);
-                    }
-                    else
-                    {
-                        breakpoint = true;
-                    }
-                }
-		delete [] evtbuf;
-	    }
-	}
-
-	// We give priority to user interrupts
-	if (gdbInterrupt)
-	    return false;
-	if (breakpoint)
-	    return true;
-    }
+    return eventLoop();
 }
 
