@@ -21,7 +21,7 @@
  *
  * This file contains the main() & support for avrjtagd.
  *
- * $Id: main.cc,v 1.40 2007/08/29 11:04:43 joerg_wunsch Exp $
+ * $Id: main.cc,v 1.51 2008/08/26 10:09:55 nilsst Exp $
  */
 
 
@@ -140,8 +140,9 @@ static void usage(const char *progname)
             "  -B, --jtag-bitrate <rate>   Set the bitrate that the JTAG box communicates\n"
             "                                with the avr target device. This must be less\n"
             "                                than 1/4 of the frequency of the target. Valid\n"
-            "                                values are 1MHz, 500KHz, 250KHz or 125KHz.\n"
-            "                                (default: 1MHz)\n");
+            "                                values are 1000/500/250/125 kHz (mkI),\n"
+	    "                                or 22 through 6400 kHz (mkII).\n"
+            "                                (default: 250 kHz)\n");
     fprintf(stderr,
 	    "  -C, --capture               Capture running program.\n"
 	    "                                Note: debugging must have been enabled prior\n"
@@ -160,7 +161,7 @@ static void usage(const char *progname)
     fprintf(stderr,
             "  -E, --event <eventlist>     List of events that do not interrupt.\n"
             "                                JTAG ICE mkII and AVR Dragon only.\n"
-            "                                Default is \"none,run,target_power_on,target_wakeup\"\n");
+            "                                Default is \"none,run,target_power_on,target_sleep,target_wakeup\"\n");
     fprintf(stderr,
 	    "  -f, --file <filename>       Specify a file for use with the --program and\n"
             "                                --verify options. If --file is passed and\n"
@@ -177,6 +178,8 @@ static void usage(const char *progname)
             "                                devices fused for compatibility.\n");
     fprintf(stderr,
 	    "  -j, --jtag <devname>        Port attached to JTAG box (default: /dev/avrjtag).\n");
+    fprintf(stderr,
+	    "  -k, --known-devices         Print a list of known devices.\n");
     fprintf(stderr,
             "  -L, --write-lockbits <ll>   Write lock bits.\n");
     fprintf(stderr,
@@ -190,6 +193,8 @@ static void usage(const char *progname)
 	    "                                option.\n");
     fprintf(stderr,
             "  -r, --read-fuses            Read fuses bytes.\n");
+    fprintf(stderr,
+            "  -R, --reset-srst            External reset through nSRST signal.\n");
     fprintf(stderr,
 	    "  -V, --version               Print version information.\n");
     fprintf(stderr,
@@ -206,6 +211,56 @@ static void usage(const char *progname)
     fprintf(stderr,
 	    "e.g. %s --erase --program --file test.bin --jtag /dev/ttyS0 :4242\n\n",
 	    progname);
+    exit(0);
+}
+
+static int comparenames(const void *a, const void *b)
+{
+    const jtag_device_def_type *ja = (const jtag_device_def_type *)a;
+    const jtag_device_def_type *jb = (const jtag_device_def_type *)b;
+
+    return strcmp(ja->name, jb->name);
+}
+
+static void knownParts()
+{
+    fprintf(stderr, "List of known AVR devices:\n\n");
+
+    jtag_device_def_type* dev = deviceDefinitions;
+    // Count the device descriptor records.
+    size_t n = 0;
+    while (dev->name != NULL)
+      n++, dev++;
+    // For historical reasons, device definitions are not
+    // sorted.  Sort them here.
+    qsort(deviceDefinitions, n, sizeof(jtag_device_def_type),
+	  comparenames);
+    fprintf(stderr,
+	    "%-15s  %10s  %8s  %8s\n", "Device Name", "Device ID",
+	    "Flash", "EEPROM");
+    fprintf(stderr,
+	    "%-15s  %10s  %8s  %8s\n", "---------------", "---------",
+	    "-------", "-------");
+    dev = deviceDefinitions;
+    while (n-- != 0)
+    {
+        unsigned eesize = dev->eeprom_page_size * dev->eeprom_page_count;
+
+	if (eesize != 0 && eesize < 1024)
+	  fprintf(stderr, "%-15s      0x%04X  %4d KiB  %4.1f KiB\n",
+		  dev->name,
+		  dev->device_id,
+		  dev->flash_page_size * dev->flash_page_count / 1024,
+		  eesize / 1024.0);
+	else
+	  fprintf(stderr, "%-15s      0x%04X  %4d KiB  %4d KiB\n",
+		  dev->name,
+		  dev->device_id,
+		  dev->flash_page_size * dev->flash_page_count / 1024,
+		  eesize / 1024);
+	dev++;
+    }
+
     exit(1);
 }
 
@@ -225,10 +280,12 @@ static struct option long_opts[] = {
     { "help",                0,       0,     'h' },
     { "ignore-intr",         0,       0,     'I' },
     { "jtag",                1,       0,     'j' },
+    { "known-devices",       0,       0,     'k' },
     { "write-lockbits",      1,       0,     'L' },
     { "read-lockbits",       0,       0,     'l' },
     { "part",                1,       0,     'P' },
     { "program",             0,       0,     'p' },
+    { "reset-srst",          0,       0,     'R' },
     { "read-fuses",          0,       0,     'r' },
     { "version",             0,       0,     'V' },
     { "verify",              0,       0,     'v' },
@@ -245,9 +302,9 @@ int main(int argc, char **argv)
     struct sockaddr_in clientname;
     struct sockaddr_in name;
     char *inFileName = 0;
-    char *jtagDeviceName = NULL;
+    const char *jtagDeviceName = NULL;
     char *device_name = 0;
-    char *eventlist = "none,run,target_power_on,target_wakeup";
+    const char *eventlist = "none,run,target_power_on,target_sleep,target_wakeup";
     unsigned long jtagBitrate = 0;
     const char *hostName = "0.0.0.0";	/* INADDR_ANY */
     int  hostPortNumber;
@@ -264,6 +321,7 @@ int main(int argc, char **argv)
     bool capture = false;
     bool verify = false;
     bool is_dragon = false;
+    bool apply_nsrst = false;
     char *progname = argv[0];
     enum {
 	MKI, MKII, MKII_DW
@@ -277,20 +335,13 @@ int main(int argc, char **argv)
     statusOut("AVaRICE version %s, %s %s\n\n",
 	      PACKAGE_VERSION, __DATE__, __TIME__);
 
-    /* If the JTAG_DEV environment variable is set, allow that to override the
-       default jtagDevicename value, but the -j option trumps the others. */
-
-    jtagDeviceName = getenv ("JTAG_DEV");
-    if (jtagDeviceName == NULL)
-        jtagDeviceName = "/dev/avrjtag";
-
     device_name = 0;
 
     opterr = 0;                 /* disable default error message */
 
     while (1)
     {
-        int c = getopt_long (argc, argv, "12B:Cc:DdeE:f:ghIj:L:lP:prVvwW:",
+        int c = getopt_long (argc, argv, "12B:Cc:DdeE:f:ghIj:kL:lP:pRrVvwW:",
                              long_opts, &option_index);
         if (c == -1)
             break;              /* no more options */
@@ -299,6 +350,8 @@ int main(int argc, char **argv)
             case 'h':
             case '?':
                 usage(progname);
+            case 'k':
+                knownParts();
 	    case '1':
 		protocol = MKI;
 		break;
@@ -318,10 +371,11 @@ int main(int argc, char **argv)
                            &units_before, &units_after,
                            &bits_before, &bits_after) != 4)
                     usage(progname);
-		if (units_before > 255 || units_after > 255 ||
-		    bits_before > 7 || bits_after > 7) {
+		if (units_before > bits_before || units_after > bits_after ||
+		    bits_before > 32 || bits_after > 32) {
 		    fprintf(stderr,
-			    "%s: daisy-chain parameters out of range\n",
+			    "%s: daisy-chain parameters out of range"
+			    " (max. 32 bits before/after)\n",
 			    progname);
 		    exit(1);
 		}
@@ -366,6 +420,9 @@ int main(int argc, char **argv)
             case 'p':
                 program = true;
                 break;
+	    case 'R':
+	        apply_nsrst = true;
+		break;
             case 'r':
                 readFuses = true;
                 break;
@@ -443,22 +500,25 @@ int main(int argc, char **argv)
         usage (progname);
     }
 
-    if (jtagBitrate == 0)
+    if (jtagBitrate == 0 && (protocol == MKI || protocol == MKII))
     {
         fprintf (stdout,
-                 "Defaulting JTAG bitrate to 1 MHz. Make sure that the target\n"
-                 "frequency is at least 4 MHz or you will likely encounter failures\n"
-                 "controlling the target.\n\n");
+                 "Defaulting JTAG bitrate to 250 kHz.\n\n");
 
-        jtagBitrate = 1000000;
+        jtagBitrate = 250000;
     }
 
     // Use a default device name to connect to if not specified on the
-    // command-line.  As the AVR Dragon can only be talked to through
-    // USB, default it to USB, but use a generic name else.
+    // command-line.  If the JTAG_DEV environment variable is set, use
+    // the name given there.  As the AVR Dragon can only be talked to
+    // through USB, default it to USB, but use a generic name else.
     if (jtagDeviceName == NULL)
     {
-      if (is_dragon)
+      char *cp = getenv ("JTAG_DEV");
+
+      if (cp != NULL)
+	jtagDeviceName = cp;
+      else if (is_dragon)
 	jtagDeviceName = "usb";
       else
 	jtagDeviceName = "/dev/avrjtag";
@@ -468,15 +528,17 @@ int main(int argc, char **argv)
 	// And say hello to the JTAG box
 	switch (protocol) {
 	case MKI:
-	    theJtagICE = new jtag1(jtagDeviceName, device_name);
+	    theJtagICE = new jtag1(jtagDeviceName, device_name, apply_nsrst);
 	    break;
 
 	case MKII:
-	    theJtagICE = new jtag2(jtagDeviceName, device_name, false, is_dragon);
+	    theJtagICE = new jtag2(jtagDeviceName, device_name, false,
+				   is_dragon, apply_nsrst);
 	    break;
 
 	case MKII_DW:
-	    theJtagICE = new jtag2(jtagDeviceName, device_name, true, is_dragon);
+	    theJtagICE = new jtag2(jtagDeviceName, device_name, true,
+				   is_dragon, apply_nsrst);
 	    break;
 	}
 
@@ -505,11 +567,18 @@ int main(int argc, char **argv)
 
     if (erase)
     {
-        theJtagICE->enableProgramming();
-	statusOut("Erasing program memory.\n");
-	theJtagICE->eraseProgramMemory();
-	statusOut("Erase complete.\n");
-        theJtagICE->disableProgramming();
+	if (protocol == MKII_DW)
+	{
+	    statusOut("WARNING: Chip erase not possible in debugWire mode; ignored\n");
+	}
+	else
+	{
+	    theJtagICE->enableProgramming();
+	    statusOut("Erasing program memory.\n");
+	    theJtagICE->eraseProgramMemory();
+	    statusOut("Erase complete.\n");
+	    theJtagICE->disableProgramming();
+	}
     }
 
     if (readFuses)
@@ -553,7 +622,7 @@ int main(int argc, char **argv)
         }
 
         theJtagICE->downloadToTarget(inFileName, program, verify);
-        theJtagICE->resetProgram();
+        theJtagICE->resetProgram(false);
     }
     else
     {
